/*
 * AnyaryOperationAbstraction.hpp
 *
 *  Created on: 20. 8. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _ANYARY_OPERATION_ABSTRACTION_HPP_
#define _ANYARY_OPERATION_ABSTRACTION_HPP_

#include <alib/memory>

#include <abstraction/ValueOperationAbstraction.hpp>
#include <common/AbstractionHelpers.hpp>

#include <registry/Registry.h>

namespace abstraction {

class AnyaryOperationAbstractionImpl : virtual public OperationAbstraction {
	ext::vector < std::pair < std::shared_ptr < OperationAbstraction >, bool > > m_params;

protected:
	ext::vector < std::pair < std::shared_ptr < OperationAbstraction >, bool > > & getParams ( ) {
		return m_params;
	}

private:
	bool attachInput ( const std::shared_ptr < OperationAbstraction > & input, size_t index, bool move, bool checkInput ) override;

	bool detachInput ( size_t index ) override;

public:
	AnyaryOperationAbstractionImpl ( ) = default;

	bool inputsAttached ( ) const override;


	bool eval ( ) override;

	size_t numberOfParams ( ) const override {
		return m_params.size ( );
	}

};

template < class ReturnType, class ParamType >
class AnyaryOperationAbstraction : virtual public AnyaryOperationAbstractionImpl, virtual public ValueOperationAbstraction < ReturnType > {
	bool checkInput ( const std::shared_ptr < OperationAbstraction > & input, size_t index ) const override {
		return abstraction::checkInput < ValueInterface < std::decay_t < ParamType > > > ( input, index );
	}

public:
	ext::type_index getParamTypeIndex ( size_t ) const override {
		return ext::type_index ( typeid ( ParamType ) );
	}

	ext::set < abstraction::ParamQualifiers::ParamQualifier > getParamTypeQualifiers ( size_t ) const override {
		return abstraction::ParamQualifiers::paramQualifiers < ParamType > ( );
	}

};

} /* namespace abstraction */

#endif /* _ANYARY_OPERATION_ABSTRACTION_HPP_ */
