/*
 * AlgorithmHelper.cpp
 *
 *  Created on: 25. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/Registry.h>
#include <exception>
#include <common/AlgorithmHelper.h>
#include <common/CastHelper.h>

namespace abstraction {

std::shared_ptr < abstraction::OperationAbstraction > AlgorithmHelper::eval ( const std::string & name, const ext::vector < std::string > & templateParams, const ext::vector < std::shared_ptr < abstraction::OperationAbstraction > > & params, std::vector < bool > & moves, AlgorithmCategories::AlgorithmCategory category ) {
	ext::vector < std::string > paramTypes;
	ext::vector < ext::set < abstraction::ParamQualifiers::ParamQualifier > > paramTypeQualifiers;
	for ( const std::shared_ptr < abstraction::OperationAbstraction > & param : params ) {
		paramTypes.push_back ( param->getReturnType ( ) );
		paramTypeQualifiers.push_back ( param->getReturnTypeQualifiers ( ) );
	}

	std::shared_ptr < abstraction::OperationAbstraction > algo = abstraction::Registry::getAlgorithmAbstraction ( name, templateParams, paramTypes, paramTypeQualifiers, category );

	unsigned i = 0;
	ext::vector < std::shared_ptr < abstraction::OperationAbstraction > > casted_params;
	for ( const std::shared_ptr < abstraction::OperationAbstraction > & param : params ) {
		if ( abstraction::Registry::isCastNoOp ( algo->getParamType ( i ), param->getReturnType ( ) ) ) {
			casted_params.push_back ( param );
		} else {
			casted_params.push_back ( abstraction::CastHelper::eval ( param, algo->getParamType ( i ), moves [ i ] ) );
			moves [ i ] = true;
		}
		++ i;
	}

	i = 0;
	for ( const std::shared_ptr < abstraction::OperationAbstraction > & param : casted_params ) {
		if ( ! algo->attachInput ( param, i, moves [ i ], true ) )
			throw std::invalid_argument ( "Can't connect param " + algo->getParamType ( i ) + " at " + ext::to_string ( i ) + " of algorithm " + name + " with result of type " + param->getReturnType ( ) + "." );
		++ i;
	}

	try {
		if ( ! algo->eval ( ) )
			throw std::invalid_argument ( "Internal error in algorithm " + name + " evaluation." );
	} catch ( const std::exception & executionException ) {
		std::throw_with_nested ( std::runtime_error ( "Evaluation of algorithm " + name + " failed." ) );
	}

	if ( abstraction::Registry::hasNormalize ( algo->getReturnType ( ) ) ) {
		std::shared_ptr < abstraction::OperationAbstraction > normalized = abstraction::Registry::getNormalizeAbstraction ( algo->getReturnType ( ) );
		if ( ! normalized->attachInput ( algo, 0, true, true ) )
			throw std::invalid_argument ( "Can't connect param at 0 of normalize of algorithm " + name + " with result of type " + algo->getReturnType ( ) + "." );
		if ( ! normalized->eval ( ) )
			throw std::invalid_argument ( "Eval of normalize of algorithm " + name + " failed." );

		algo = normalized;
	}

	return algo;
}


} /* namespace abstraction */
