/*
 * ImmediateRegistry.cpp
 *
 *  Created on: 21. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/ImmediateRegistry.hpp>

namespace abstraction {

ext::map < std::string, std::unique_ptr < ImmediateRegistry::Entry > > & ImmediateRegistry::getEntries ( ) {
	static ext::map < std::string, std::unique_ptr < Entry > > fileWriters;
	return fileWriters;
}

void ImmediateRegistry::unregisterImmediate ( const std::string & result ) {
	if ( getEntries ( ).erase ( result ) == 0u )
		throw std::invalid_argument ( "Entry " + result + " not registered." );
}

void ImmediateRegistry::registerImmediate ( std::string result, std::unique_ptr < Entry > entry ) {
	auto iter = getEntries ( ).insert ( std::make_pair ( std::move ( result ), std::move ( entry ) ) );
	if ( ! iter.second )
		throw std::invalid_argument ( "Entry " + iter.first->first + " already registered." );
}

std::shared_ptr < abstraction::OperationAbstraction > ImmediateRegistry::getAbstraction ( const std::string & result, std::string value ) {
	auto res = getEntries ( ).find ( result );
	if ( res == getEntries ( ).end ( ) )
		throw std::invalid_argument ( "Entry " + result + " not available." );

	return res->second->getAbstraction ( std::move ( value ) );
}

} /* namespace abstraction */
